@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -16,6 +16,7 @@ package plugin
 
 import (
 	"context"
+	"crypto/tls"
 	"sync"
 	"time"
 
@@ -228,7 +229,7 @@ func NewConnManager(
 
 // GetConnection returns an existing connection or creates a new one.
 func (c *ConnManager) GetConnection(
-	connURI *uri.URI,
+	connURI uri.URI, //nolint:gocritic
 	params map[string]string,
 ) (*MongoConn, error) {
 	ck := createConnKey(connURI, params)
@@ -240,7 +241,7 @@ func (c *ConnManager) GetConnection(
 		return conn, nil
 	}
 
-	conn, err := c.create(ck)
+	conn, err := c.create(ck, params)
 	if err != nil {
 		return nil, errs.Wrap(err, "failed to create new connection")
 	}
@@ -250,11 +251,11 @@ func (c *ConnManager) GetConnection(
 
 // getConn returns a connection with given uri if it exists and also updates
 // lastTimeAccess, otherwise returns nil.
-func (c *ConnManager) getConn(ck *connKey) *MongoConn {
+func (c *ConnManager) getConn(ck connKey) *MongoConn { //nolint:gocritic
 	c.connectionsMu.Lock()
 	defer c.connectionsMu.Unlock()
 
-	conn, ok := c.connections[*ck]
+	conn, ok := c.connections[ck]
 	if !ok {
 		return nil
 	}
@@ -265,13 +266,13 @@ func (c *ConnManager) getConn(ck *connKey) *MongoConn {
 }
 
 func (c *ConnManager) setConn(
-	ck *connKey,
+	ck connKey, //nolint:gocritic
 	conn *MongoConn,
 ) *MongoConn {
 	c.connectionsMu.Lock()
 	defer c.connectionsMu.Unlock()
 
-	existingConn, ok := c.connections[*ck]
+	existingConn, ok := c.connections[ck]
 	if ok {
 		err := closeSession(context.Background(), conn.session)
 		if err != nil {
@@ -283,7 +284,7 @@ func (c *ConnManager) setConn(
 		return existingConn
 	}
 
-	c.connections[*ck] = conn
+	c.connections[ck] = conn
 
 	return conn
 }
@@ -340,9 +341,10 @@ func (c *ConnManager) housekeeper(ctx context.Context, interval time.Duration) {
 
 // create creates a new connection with given credentials.
 func (c *ConnManager) create(
-	ck *connKey,
+	ck connKey, //nolint:gocritic
+	params map[string]string,
 ) (*MongoConn, error) {
-	opt, err := c.createOptions(ck)
+	opt, err := c.createOptions(ck.uri, params)
 	if err != nil {
 		return nil, err
 	}
@@ -393,33 +395,32 @@ func (c *ConnManager) create(
 
 // get returns a connection with given uri if it exists and also updates lastTimeAccess, otherwise returns nil.
 func (c *ConnManager) createOptions(
-	ck *connKey,
+	connURI uri.URI, //nolint:gocritic
+	params map[string]string,
 ) (*options.ClientOptions, error) {
+	details, err := createTLS(params)
+	if err != nil {
+		return nil, err
+	}
+
 	opt := options.Client()
 
-	if ck.uri.User() != "" {
+	if connURI.User() != "" {
 		creds := options.Credential{}
-		creds.Username = ck.uri.User()
-		creds.Password = ck.uri.Password()
+		creds.Username = connURI.User()
+		creds.Password = connURI.Password()
 		creds.PasswordSet = true
 		opt = opt.SetAuth(creds)
 	}
 
-	if ck.tlsConnect != string(tlsconfig.Disabled) {
-		details, err := createTLS(ck)
+	if details.TlsConnect != disable {
+		err := c.setTLSConfig(opt, details)
 		if err != nil {
 			return nil, err
 		}
-
-		tlsConfig, err := details.GetTLSConfig()
-		if err != nil {
-			return nil, errs.Wrap(err, "failed to get TLS config")
-		}
-
-		opt = opt.SetTLSConfig(tlsConfig)
 	}
 
-	opt.SetHosts([]string{ck.uri.Addr()})
+	opt.SetHosts([]string{connURI.Addr()})
 	opt.SetDirect(true)
 	opt.SetConnectTimeout(c.timeout)
 	opt.SetServerSelectionTimeout(c.timeout)
@@ -428,41 +429,39 @@ func (c *ConnManager) createOptions(
 	return opt, nil
 }
 
-func createTLS(ck *connKey) (*tlsconfig.Details, error) {
+func createTLS(params map[string]string) (*tlsconfig.Details, error) {
 	var (
 		validateCA     = true
 		validateClient = false
+		tlsType        = params[tlsConnectParam]
 	)
 
-	connectionType, err := tlsconfig.NewTLSConnectionType(ck.tlsConnect)
-	if err != nil {
-		return nil, errs.Wrap(err, "failed to parse TLS connection type")
+	if tlsType == "" {
+		tlsType = disable
 	}
 
 	details := tlsconfig.NewDetails(
 		"",
-		ck.rawUri,
-		tlsconfig.WithTLSCaFile(ck.tlsCA),
-		tlsconfig.WithTLSCertFile(ck.tlsCert),
-		tlsconfig.WithTLSKeyFile(ck.tlsKey),
-		tlsconfig.WithTLSConnect(connectionType),
-		tlsconfig.WithAllowedConnections(
-			string(tlsconfig.Disabled),
-			require,
-			verifyCa,
-			verifyFull,
-		),
+		tlsType,
+		params[tlsCAParam],
+		params[tlsCertParam],
+		params[tlsKeyParam],
+		params[uriParam],
+		disable,
+		require,
+		verifyCa,
+		verifyFull,
 	)
 
-	if ck.tlsConnect == string(tlsconfig.Disabled) || ck.tlsConnect == require {
+	if tlsType == disable || tlsType == require {
 		validateCA = false
 	}
 
-	if details.TLSKeyFile != "" || details.TLSCertFile != "" {
+	if details.TlsKeyFile != "" || details.TlsCertFile != "" {
 		validateClient = true
 	}
 
-	err = details.Validate(validateCA, validateClient, validateClient)
+	err := details.Validate(validateCA, validateClient, validateClient)
 	if err != nil {
 		return nil, zbxerr.ErrorInvalidConfiguration.Wrap(err)
 	}
@@ -470,6 +469,59 @@ func createTLS(ck *connKey) (*tlsconfig.Details, error) {
 	return &details, nil
 }
 
+func (c *ConnManager) setTLSConfig(
+	opt *options.ClientOptions,
+	details *tlsconfig.Details,
+) error {
+	var cfg *tls.Config
+	var err error
+
+	switch details.TlsConnect {
+	case "required":
+		cfg, err = c.getRequiredTLSConfig(details)
+		if err != nil {
+			return err
+		}
+	case "verify_ca":
+		cfg, err = details.GetTLSConfig(true)
+		if err != nil {
+			return errs.Wrap(err, "failed to get TLS config for verify_ca connection")
+		}
+
+		cfg.VerifyPeerCertificate = tlsconfig.VerifyPeerCertificateFunc(
+			"",
+			cfg.RootCAs,
+		)
+	case "verify_full":
+		cfg, err = details.GetTLSConfig(false)
+		if err != nil {
+			return errs.Wrap(err, "failed to get TLS config for verify_full connection")
+		}
+	}
+
+	opt.SetTLSConfig(cfg)
+
+	return nil
+}
+
+func (c *ConnManager) getRequiredTLSConfig(
+	details *tlsconfig.Details,
+) (*tls.Config, error) {
+	if details.TlsCaFile != "" {
+		c.log.Warningf(
+			"server CA will not be verified for %s",
+			details.TlsConnect,
+		)
+	}
+
+	clientCerts, err := details.LoadCertificates()
+	if err != nil {
+		return nil, err
+	}
+
+	return &tls.Config{Certificates: clientCerts, InsecureSkipVerify: true}, nil
+}
+
 // updateAccessTime updates the last time a connection was accessed.
 func (conn *MongoConn) updateAccessTime() {
 	conn.lastTimeAccess = time.Now()
@@ -494,14 +546,14 @@ func closeSession(ctx context.Context, session mongo.Session) error {
 	return nil
 }
 
-func createConnKey(u *uri.URI, params map[string]string) *connKey {
+func createConnKey(uri uri.URI, params map[string]string) connKey {
 	tlsType := params[tlsConnectParam]
-	if tlsType == "" || tlsType == disable {
-		tlsType = string(tlsconfig.Disabled)
+	if tlsType == "" {
+		tlsType = disable
 	}
 
-	return &connKey{
-		uri:        *u,
+	return connKey{
+		uri:        uri,
 		rawUri:     params[uriParam],
 		tlsConnect: tlsType,
 		tlsCA:      params[tlsCAParam],
